/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 */
package org.komodo.rest.relational.json;

import java.io.IOException;
import org.komodo.rest.relational.AbstractKomodoContentAttribute;
import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;

/**
 * A GSON serializer/deserializer for {@status AbstractKomodoContentAttribute}.
 */
public abstract class AbstractContentSerializer<T extends AbstractKomodoContentAttribute> extends TypeAdapter<T> {

    protected String readContent(final JsonReader in, final AbstractKomodoContentAttribute contentAttr, final String name)
        throws IOException {
        if (AbstractKomodoContentAttribute.CONTENT_LABEL.equals(name)) {
            contentAttr.setContent(in.nextString());
            return name;
        }

        return null;
    }

    /**
     * {@inheritDoc}
     *
     * @see com.google.gson.TypeAdapter#read(com.google.gson.stream.JsonReader)
     */
    @Override
    public abstract T read(final JsonReader in) throws IOException;

    protected void writeContent(final JsonWriter out, final AbstractKomodoContentAttribute value) throws IOException {
        out.name(AbstractKomodoContentAttribute.CONTENT_LABEL);
        out.value(value.getContent());
    }

    /**
     * {@inheritDoc}
     *
     * @see com.google.gson.TypeAdapter#write(com.google.gson.stream.JsonWriter, java.lang.Object)
     */
    @Override
    public abstract void write(final JsonWriter out, final T value) throws IOException;
}
